<#
* Copyright(c) 2006-2016 ThreatSTOP, Inc. All Rights Reserved
* NOTICE: All information contained herein is, and remains the property of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and 
* technical concepts contained herein are proprietary to ThreatSTOP, Inc. and its suppliers and certain aspects thereof are protected by United 
* States Patent No. 8,533,822 and United States Patent No. 8,869,237, and are protected by trade secret or copyright law.
* US Government Entities: The ThreatSTOP service, software and documentation, as applicable, are "commercial computer software" and
* "commercial computer software documentation" developed exclusively at private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
* 12.212 or DFARS 227 7202 and their successors, as applicable, use, reproduction and disclosure of the software and documentation is
* governed by the terms of Threatstop's commercial agreement.
#>
# ErrID start: 1
#
# 1. export DNS errors to log file. export DNS errors to log file. split into chunks if necessary
# 2. truncate DNS analytical log
# 3. setup.ps1 will schedule this script to execute every n minutes (get value from config file)

$ErrorActionPreference = "Stop"
#set up event log source for ThreatSTOP
if (![System.Diagnostics.EventLog]::SourceExists("ThreatSTOP"))
{
    New-EventLog -LogName Application -Source "ThreatSTOP"
}

#Load config values into hash table:
$scriptPath = split-path -parent $MyInvocation.MyCommand.Definition
$iniPath = (get-item $scriptPath).parent.fullname
. "$scriptPath\functions.ps1"
Get-Content "$iniPath\ThreatSTOP.ini" | foreach-object -begin {$h=@{}} -process { $k = [regex]::split($_,'='); if(($k[0].CompareTo("") -ne 0) -and ($k[0].StartsWith("[") -ne $True -and $k[0].StartsWith("#")  -ne $True)) { $h.Add($k[0].Trim(), $k[1].Trim()) } }
$LogFolder = $h.LogFolder

#create Log folder if missing
if (!(Test-Path $LogFolder))
{
    New-Item -ItemType Directory -Force -Path $LogFolder
}

try
{
    $Log = $null
	$FilePath = $LogFolder + '\DNSPolicy.' + $(((get-date).ToUniversalTime()).ToString("yyyyMMddTHHmmssZ")) + '.log'
	$Log = Get-WinEvent -FilterHashtable @{Logname='Microsoft-Windows-DNSServer/Analytical';id=258} -Oldest -ErrorAction Stop | where {$_.Message -match "PolicyName=Threat"}
}
catch
{
	#no events found?
	$errmsg = 'DNS Policy Log export: ' + $_.Exception.Message
	Write-EventLog -LogName Application -Source 'ThreatSTOP'-EntryType Information -EventId 1 -Message $errmsg 
}
try
{
    if ($Log) {$Log | Export-Csv $FilePath -ErrorAction Stop}
}
catch
{
	Write-EventLog -LogName Application -Source 'ThreatSTOP' -EntryType Warning -EventId 2 -Message $_.Exception.Message
}
if (Test-Path $FilePath)
{
	#split file into chunks if necessary
	if ((Get-Item $FilePath).length -gt 14mb)
	{
		Split-FileByLine -FileName $FilePath -OutPath $LogFolder -MaxFileSize 14MB
		Remove-Item $FilePath
	}

	try
	{
		#clear logs by restarting analytic log
		echo y | wevtutil set-log "Microsoft-Windows-DNSServer/Analytical" /enabled:false  
		echo y | wevtutil set-log "Microsoft-Windows-DNSServer/Analytical" /enabled:true #this command also clears log
	}
	catch
	{
		Write-EventLog -LogName Application -Source 'ThreatSTOP' -EntryType Warning -EventId 3 -Message $_.Exception.Message
	}
}

